@@ -1,7 +1,7 @@
 /* Output the generated parsing program for bison,
 
-   Copyright (C) 2000-2003, 2006-2007, 2009-2013 Free Software
-   Foundation, Inc.
+   Copyright (C) 2000-2003, 2006-2007, 2009-2015, 2018-2021 Free
+   Software Foundation, Inc.
 
    This file is part of Bison, the GNU Compiler Compiler.
 
@@ -23,6 +23,5 @@
 
 /* Output the parsing tables and the parser code to FTABLE.  */
 void output (void);
-char const *compute_pkgdatadir (void);
 
 #endif /* !OUTPUT_H_ */
