@@ -1,7 +1,7 @@
 /* Keeping a unique copy of strings.
 
-   Copyright (C) 2002-2003, 2008-2015, 2018 Free Software Foundation,
-   Inc.
+   Copyright (C) 2002-2003, 2008-2015, 2018-2021 Free Software
+   Foundation, Inc.
 
    This file is part of Bison, the GNU Compiler Compiler.
 
